import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, scaleTime, timeFormat, extent, line, curveNatural, curveBasis } from 'd3';

const width = 900;
const height = 500;
const margin = {
  top: 15,
  right: 35,
  bottom: 55,
  left: 75,
};
const xAxisLabelOffset = 60;
const yAxisLabelOffset = 60;

// years on X-axis
const xValue = (d) => d.years;
const xAxisLabel = 'years';

// pOpulatiOn on Y-axis
const yValue = (d) => d.population_hyderabad;
const yAxisLabel = 'population';

const xAxisTickFormat = timeFormat("%Y");

const fetchCSVData = () => {
  
  const csvUrl =
  'https://gist.githubusercontent.com/abhi051197/1e8efa9ed5b572ed2226861b7d24ab45/raw/4d4482aae53b2da5b4b1b8d58456569df761d6e6/ice7q3.csv';

  
  const [data, setData] = useState(null);

  useEffect(() => {
    
    // Formats each item over data.
    const row = (d) => {
      d.years = new Date(d.years),
      d.population_hyderabad = +d.population_hyderabad
      return d;
    };
    
  	// Formats data depending on years
    csv(csvUrl, row).then((data) => {
			setData(data
							.sort(function(a, b) {
			return d3.ascending(a.years, b.years)
							})
			);
    });
  }, []);
  return data;
};

// Render line b/w axes
const LineChart = ({
  data,
  xScale,
  yScale,
  xValue,
  yValue,
  tooltipFormat
}) => {
    
	return(
  	<g className="marks">
    <path
      fill="none"
      stroke="brown"
      d={
        line()
        .x(d => xScale(xValue(d)))
        .y(d => yScale(yValue(d)))
        .curve(curveBasis)
        (data)
        }
    />
  </g>
  );
};

// Renders Y axis depending on y scale
const YAxis = ({ yScale, innerWidth, tickOffset = 3 }) =>
  yScale.ticks().map((tickValue) => (
    <g className="tick" transform={`translate(0,${yScale(tickValue)})`}>
      <line x2={innerWidth} />
      <text
        key={tickValue}
        style={{ textAnchor: 'end' }}
        x={-tickOffset}
        dy=".32em"
        y={yScale(tickValue)}
      >
        {tickValue}
      </text>
    </g>
  ));

// Renders X axis depending on x scale
const XAxis = ({ xScale, innerHeight, tickFormat, tickOffset = 0 }) =>
  xScale.ticks().map((tickValue) => (
    <g className="tick"
      key={tickValue}
      transform={`translate(${xScale(tickValue)},0)`}>
      <line y2={innerHeight}/>
      <text
        style={{ textAnchor: 'middle' }}
        dy=".71em"
        y={innerHeight + tickOffset}
      >
        {tickFormat(tickValue)}
      </text>
    </g>
  ));


// actual component that graphs
const App = () => {
  
  // csv data from uri
  const data = fetchCSVData();

  if (!data) {
    return <pre>Loading data...</pre>;
  }  
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  // Creating  scale for X-axis depending over x value
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
  	.nice();
  
  // Creating scale for Y-axis depending on y-Value
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
  	.nice();

  return (
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <XAxis
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={7}
        />
        <YAxis
          yScale={yScale}
          innerWidth={innerWidth}
        />
				<LineChart
          data={data}
          xScale={xScale}
          yScale={yScale}
          xValue={xValue}
          yValue={yValue}
          tooltipFormat={xAxisTickFormat}
          circleRadius={3}
        />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
          tickOffset={7}
        >
          {xAxisLabel}
        </text>
        <text
          className="axis-label"
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset},${innerHeight / 2}) rotate(-90)`}
        >
          {yAxisLabel}
        </text>
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
